@@ -38,6 +38,8 @@ from hypothesis.internal.compat import (
     text_type,
     typing_root_type,
 )
+from hypothesis.searchstrategy.lazy import unwrap_strategies
+from hypothesis.searchstrategy.strategies import OneOfStrategy
 
 
 def type_sorting_key(t):
@@ -83,10 +85,18 @@ def is_a_type(thing):
     )
 
 
+def is_typing_literal(thing):
+    return (
+        hasattr(typing, "Literal")
+        and getattr(thing, "__origin__", None) == typing.Literal
+    )
+
+
 def from_typing_type(thing):
     # We start with special-case support for Union and Tuple - the latter
-    # isn't actually a generic type.  Support for Callable may be added to
-    # this section later.
+    # isn't actually a generic type. Then we handle Literal since it doesn't
+    # support `isinstance`. Support for Callable may be added to this section
+    # later.
     # We then explicitly error on non-Generic types, which don't carry enough
     # information to sensibly resolve to strategies at runtime.
     # Finally, we run a variation of the subclass lookup in st.from_type
@@ -115,12 +125,32 @@ def from_typing_type(thing):
         elif len(elem_types) == 1 and elem_types[0] == ():
             return st.tuples()  # Empty tuple; see issue #1583
         return st.tuples(*map(st.from_type, elem_types))
+    if is_typing_literal(thing):  # pragma: no cover  # new in Python 3.8
+        args_dfs_stack = list(thing.__args__)
+        literals = []
+        while args_dfs_stack:
+            arg = args_dfs_stack.pop()
+            if is_typing_literal(arg):
+                args_dfs_stack.extend(reversed(arg.__args__))
+            else:
+                literals.append(arg)
+        return st.sampled_from(literals)
     if isinstance(thing, typing.TypeVar):
         if getattr(thing, "__bound__", None) is not None:
-            return st.from_type(thing.__bound__)
+            strat = unwrap_strategies(st.from_type(thing.__bound__))
+            if not isinstance(strat, OneOfStrategy):
+                return strat
+            # The bound was a union, or we resolved it as a union of subtypes,
+            # so we need to unpack the strategy to ensure consistency across uses.
+            # This incantation runs a sampled_from over the strategies inferred for
+            # each part of the union, wraps that in shared so that we only generate
+            # from one type per testcase, and flatmaps that back to instances.
+            return st.shared(
+                st.sampled_from(strat.original_strategies), key="typevar=%r" % (thing,)
+            ).flatmap(lambda s: s)
         if getattr(thing, "__constraints__", None):
             return st.shared(
-                st.sampled_from(thing.__constraints__), key="typevar-with-constraint"
+                st.sampled_from(thing.__constraints__), key="typevar=%r" % (thing,)
             ).flatmap(st.from_type)
         # Constraints may be None or () on various Python versions.
         return st.text()  # An arbitrary type for the typevar
